@@ -1,4 +1,4 @@
-// Copyright (C) 2001,2002 Federico Montesino Pouzols <fedemp@altern.org>.
+// Copyright (C) 2001,2002,2004 Federico Montesino Pouzols <fedemp@altern.org>.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -14,29 +14,26 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 // 
-// As a special exception to the GNU General Public License, permission is 
-// granted for additional uses of the text contained in its release 
-// of ccRTP.
-// 
-// The exception is that, if you link the ccRTP library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the ccRTP library code into it.
-// 
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name ccRTP.  If you copy code from other releases into a copy of
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however    
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.    
+//
+// This exception applies only to the code released under the name GNU
+// ccRTP.  If you copy code from other releases into a copy of GNU
 // ccRTP, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for ccRTP, it is your choice
+//
+// If you write modifications of your own for GNU ccRTP, it is your choice
 // whether to permit this exception to apply to your modifications.
-// If you do not wish that, delete this exception notice.  
+// If you do not wish that, delete this exception notice.
+//
 
 /** 
  * @file oqueue.h 
@@ -66,7 +63,7 @@ namespace ost {
  *
  * @author Federico Montesino Pouzols <fedemp@altern.org>
  **/
-class CCXX_CLASS_EXPORT DestinationListHandler
+class __EXPORT DestinationListHandler
 {
 protected:
 	struct TransportAddress;
@@ -147,10 +144,9 @@ private:
  *
  * A generic outgoing RTP data queue supporting multiple destinations.
  *
- * @todo Add CSRC interface.
  * @author Federico Montesino Pouzols <fedemp@altern.org>
  **/
-class CCXX_CLASS_EXPORT OutgoingDataQueue:
+class __EXPORT OutgoingDataQueue:
 	public OutgoingDataQueueBase,
 	protected DestinationListHandler
 {
@@ -175,6 +171,20 @@ public:
 			  tpport_t dataPort = DefaultRTPDataPort,
 			  tpport_t controlPort = 0);
 
+	/**
+	 * Add csrc as the CSRC identifier of a new contributor. This
+	 * method adds the CSRC identifier to a list of contributors
+	 * that will be inserted in every packet enqueued from now on.
+	 **/
+	void
+	addContributor(uint32 csrc);
+
+	/**
+	 * Remove CSRC from the list of contributors.
+	 **/
+	bool
+	removeContributor(uint32 csrc);
+
  	/**
  	 * Determine if outgoing packets are waiting to send.
  	 *
@@ -412,7 +422,7 @@ private:
 /** @}*/ // oqueue
 
 #ifdef  CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif  //CCXX_RTP_OQUEUE_H_
