@@ -14,29 +14,26 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 // 
-// As a special exception to the GNU General Public License, permission is 
-// granted for additional uses of the text contained in its release 
-// of ccRTP.
-// 
-// The exception is that, if you link the ccRTP library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the ccRTP library code into it.
-// 
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name ccRTP.  If you copy code from other releases into a copy of
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however    
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.    
+//
+// This exception applies only to the code released under the name GNU
+// ccRTP.  If you copy code from other releases into a copy of GNU
 // ccRTP, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for ccRTP, it is your choice
+//
+// If you write modifications of your own for GNU ccRTP, it is your choice
 // whether to permit this exception to apply to your modifications.
-// If you do not wish that, delete this exception notice.  
+// If you do not wish that, delete this exception notice.
+//
 
 #ifndef	CCXX_RTP_RTPPKT_H_
 #define CCXX_RTP_RTPPKT_H_
@@ -71,7 +68,7 @@ namespace ost {
  *
  * @author David Sugar <dyfet@ostel.com>
  **/
-class  CCXX_CLASS_EXPORT RTPPacket
+class  __EXPORT RTPPacket
 {
 private:
 	struct RTPFixedHeader;
@@ -123,7 +120,7 @@ public:
 	{ return (uint8*)(buffer + getHeaderSize()); }
 		
 	/**
-	 * @return lenght of the payload section, in octets.
+	 * @return length of the payload section, in octets.
 	 **/
 	inline uint32
 	getPayloadSize() const
@@ -322,7 +319,7 @@ protected:
          uint32 fixsize = sizeof(RTPFixedHeader) + (getHeader()->cc << 2); 
 	 return (reinterpret_cast<RTPHeaderExt*>(buffer + fixsize));
 	}
-		
+
 	/**
 	 * Obtain the absolute timestamp carried in the packet header.
 	 *
@@ -387,6 +384,31 @@ private:
 		uint32 timestamp;       ///< timestamp
 		uint32 sources[1];      ///< contributing sources
 	};
+
+	/**
+ 	 * @struct RFC2833Payload
+	 * @short a structure defining RFC2833 Telephony events.
+	 *
+	 * structure to define RFC2833 telephony events in RTP.  You can
+	 * use this by recasing the pointer returned by getPayload().
+	 */
+
+	struct RFC2833Payload
+	{
+#if __BYTE_ORDER == __BIG_ENDIAN
+        	uint8 event : 8;
+        	bool ebit : 1;
+        	bool rbit : 1;
+        	uint8 vol : 6;
+        	uint16 duration : 16;
+#else
+        	uint8 event : 8;
+        	uint8 vol : 6;
+        	bool rbit : 1;
+        	bool ebit : 1; 
+        	uint16 duration : 16;
+#endif
+	};	
 	
 	/**
 	 * @struct RTPHeaderExt
@@ -401,6 +423,32 @@ private:
 		uint16 length;    ///< number of 32-bit words in the extension
 	};
 #pragma pack()
+
+	/* definitions for access to most common 2833 fields... */
+
+	/**
+	 * Fetch a raw 2833 packet.
+	 *
+	 * @return low level 2833 data structure.
+	 */
+	inline struct RFC2833Payload *getRaw2833Payload(void)
+		{return (struct RFC2833Payload *)getPayload();};
+
+	/**
+	 * Fetch 2833 duration field.
+	 *
+	 * @return 2833 duration in native host machine byte order.
+	 */
+	inline uint16 get2833Duration(void)
+		{return ntohs(getRaw2833Payload()->duration);};
+
+	/**
+	 * Set 2833 duration field.
+	 *
+	 * @param timestamp to use, native host machine byte order.
+	 */
+	inline void set2833Duration(uint16 d)
+		{getRaw2833Payload()->duration = htons(d);};
 };
 
 /**
@@ -413,7 +461,7 @@ private:
  *
  * @author Federico Montesino Pouzols <fedemp@altern.org> 
  **/
-class CCXX_CLASS_EXPORT OutgoingRTPPkt : public RTPPacket
+class __EXPORT OutgoingRTPPkt : public RTPPacket
 {
 public:
 	/**
@@ -572,7 +620,7 @@ private:
  *
  * @author Federico Montesino Pouzols <fedemp@altern.org> 
  */
-class CCXX_CLASS_EXPORT IncomingRTPPkt : public RTPPacket
+class __EXPORT IncomingRTPPkt : public RTPPacket
 {
 public:
 	/**
@@ -656,7 +704,7 @@ private:
 /** @}*/ // rtppacket
 
 #ifdef  CCXX_NAMESPACES
-};
+}
 #endif
 
 #endif  // ndef CCXX_RTP_RTPPKT_H_
