@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2002 Open Source Telecom Corporation.
+// Copyright (C) 1999-2004 Open Source Telecom Corporation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -14,29 +14,26 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 // 
-// As a special exception to the GNU General Public License, permission is
-// granted for additional uses of the text contained in its release 
-// of ccRTP.
-// 
-// The exception is that, if you link the ccRTP library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the ccRTP library code into it.
-// 
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name ccRTP.  If you copy code from other releases into a copy of
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however    
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.    
+//
+// This exception applies only to the code released under the name GNU
+// ccRTP.  If you copy code from other releases into a copy of GNU
 // ccRTP, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for ccRTP, it is your choice
+//
+// If you write modifications of your own for GNU ccRTP, it is your choice
 // whether to permit this exception to apply to your modifications.
-// If you do not wish that, delete this exception notice.  
+// If you do not wish that, delete this exception notice.
+//
 
 /**
  * @file control.cpp
@@ -264,7 +261,6 @@ QueueRTCPManager::timerReconsideration()
 void 
 QueueRTCPManager::expireSSRCs()
 {
-	// TODO
 }
 
 void
@@ -458,9 +454,9 @@ QueueRTCPManager::onGotRR(SyncSource& source, RecvReport& RR, uint8 blocks)
 void
 QueueRTCPManager::updateAvgRTCPSize(size_t len)
 {
-	uint32 newlen = len; 
+	size_t newlen = len; 
 	newlen += lowerHeadersSize;
-	rtcpAvgSize = ( (15 * rtcpAvgSize) >> 4 ) + ( newlen >> 4);
+	rtcpAvgSize = (uint16)(( (15 * rtcpAvgSize) >> 4 ) + ( newlen >> 4));
 }
 
 bool
@@ -473,13 +469,16 @@ QueueRTCPManager::getBYE(RTCPPacket& pkt, size_t& pointer, size_t)
 
 	if ( (sizeof(RTCPFixedHeader) + pkt.fh.block_count * sizeof(uint32))
 	     < pkt.getLength() ) {
-		uint16 endpointer = pointer + sizeof(RTCPFixedHeader) + 
-			pkt.fh.block_count * sizeof(uint32);
+		uint16 endpointer = (uint16)(pointer + sizeof(RTCPFixedHeader) + 
+			pkt.fh.block_count * sizeof(uint32));
 		uint16 len = rtcpRecvBuffer[endpointer];
 		reason = new char[len + 1];
 		memcpy(reason,rtcpRecvBuffer + endpointer + 1,len);
 		reason[len] = '\0';
-	} 
+	} else { // avoid dangerous conversion of NULL to a C++ string.
+		reason = new char[1];
+		reason[0] = '\0';
+	}
 
 	int i = 0;
 	while ( i < pkt.fh.block_count ){
@@ -573,21 +572,30 @@ QueueRTCPManager::onGotSDESChunk(SyncSource& source, SDESChunk& chunk,
 	SyncSourceLink* srcLink = getLink(source);
 	Participant* part = source.getParticipant();
 
-	size_t pointer = sizeof(chunk.ssrc);;
+	size_t pointer = sizeof(chunk.ssrc);
 
 	// process chunk items
 	while ( (pointer < len) && !end ) {
 		SDESItem* item = 
-			reinterpret_cast<SDESItem*>(&(chunk) + pointer);
+			reinterpret_cast<SDESItem*>(size_t(&(chunk)) + pointer);
 		if ( item->type > SDESItemTypeEND && item->type <= SDESItemTypeLast) {
 			pointer += sizeof(item->type) + sizeof(item->len) + 
 				item->len;
 			if ( NULL == part && SDESItemTypeCNAME == item->type ) {
-				part = new Participant("-");
+				const RTPApplication& app = getApplication();
+				std::string cname = std::string(item->data,item->len);
+				const Participant* p = app.getParticipant(cname);
+				if ( p ) {
+					part = const_cast<Participant*>(p);
+					setParticipant(*(srcLink->getSource()),*part);
+				} else {
+					part = new Participant("-");
+					addParticipant(const_cast<RTPApplication&>(getApplication()),*part);
+				}
 				setParticipant(*(srcLink->getSource()),*part);
-				addParticipant(const_cast<RTPApplication&>(getApplication()),*part);
 			}
-			
+
+			// support for CNAME updates
 			if ( part )
 				setSDESItem(part,(SDESItemType)item->type,
 					    item->data,item->len);
@@ -630,7 +638,7 @@ timeval
 QueueRTCPManager::computeRTCPInterval()
 {	
 	float bwfract = controlBwFract * getSessionBandwidth();
-	uint32 participants = 0;
+	uint32 participants = getMembersCount();
 	if ( getSendersCount() > 0 &&
 	     ( getSendersCount() < (getMembersCount() * sendControlBwFract) )) {
 		// reserve "sendControlBwFract" fraction of the total
@@ -651,13 +659,21 @@ QueueRTCPManager::computeRTCPInterval()
 	if ( rtcpInitial )
 		min_interval /= 2;
 	// this is the real computation:
-	microtimeout_t interval = static_cast<microtimeout_t>
-		((participants * rtcpAvgSize / bwfract) * 1000000);
-	if ( interval < rtcpMinInterval )
-		interval = rtcpMinInterval;
-	
-	interval *= static_cast<microtimeout_t>(0.5 + 
-						(rand() / (RAND_MAX + 1.0)));
+	microtimeout_t interval = 0;
+	if ( bwfract != 0 ) {
+		interval = static_cast<microtimeout_t>
+			((participants * rtcpAvgSize / bwfract) * 1000000);
+
+		if ( interval < rtcpMinInterval )
+			interval = rtcpMinInterval;
+	} else {
+		// 100 seconds instead of infinite
+		interval = 100000000;
+	}
+
+	interval = static_cast<microtimeout_t>(interval * ( 0.5 + 
+						(rand() / (RAND_MAX + 1.0))));
+
 	timeval result;
 	result.tv_sec = interval / 1000000;
 	result.tv_usec = interval % 1000000;
@@ -667,7 +683,7 @@ QueueRTCPManager::computeRTCPInterval()
 size_t
 QueueRTCPManager::dispatchBYE(const std::string& reason)
 {
-	// for this method, see section 6.3.7 in RFC XXXX
+	// for this method, see section 6.3.7 in RFC 3550
 	// never send a BYE packet if never sent an RTP or RTCP packet
 	// before
 	if ( !(getSendPacketCount() || getSendRTCPPacketCount()) )
@@ -682,9 +698,9 @@ QueueRTCPManager::dispatchBYE(const std::string& reason)
 		setPrevMembersNum(1);
 		rtcpInitial = true;
 		rtcpWeSent = false;
-		rtcpAvgSize = sizeof(RTCPFixedHeader) + sizeof(uint32) +
+		rtcpAvgSize = (uint16)(sizeof(RTCPFixedHeader) + sizeof(uint32) +
 			strlen(reason.c_str()) + 
-			(4 - (strlen(reason.c_str()) & 0x03));
+			(4 - (strlen(reason.c_str()) & 0x03)));
 		gettimeofday(&(reconsInfo.rtcpTc),NULL);
 		timeval T = computeRTCPInterval();
 		timeradd(&(reconsInfo.rtcpTp),&T,&(reconsInfo.rtcpTn));
@@ -698,8 +714,9 @@ QueueRTCPManager::dispatchBYE(const std::string& reason)
 
 
 	unsigned char buffer[500];
-	// Build an empty RR as first packet in the compound.
-	// TODO: provide more information if available.
+	// Build an empty RR as first packet in the compound.  
+        // TODO: provide more information if available. Not really
+	// important, since this is the last packet being sent.
 	RTCPPacket* pkt = reinterpret_cast<RTCPPacket*>(buffer);
 	pkt->fh.version = CCRTP_VERSION;
 	pkt->fh.padding = 0;
@@ -720,7 +737,7 @@ QueueRTCPManager::dispatchBYE(const std::string& reason)
 	len += sizeof(RTCPFixedHeader) + sizeof(BYEPacket);
 	// add the optional reason
 	if ( reason.c_str() != NULL ){
-		pkt->info.BYE.length = strlen(reason.c_str());
+		pkt->info.BYE.length = (uint8)strlen(reason.c_str());
 		memcpy(buffer + len,reason.c_str(),pkt->info.BYE.length);
 		len += pkt->info.BYE.length;
 		padlen = 4 - ((len - len1) & 0x03);
@@ -770,7 +787,7 @@ QueueRTCPManager::getOnlyBye()
 					getSourceBySSRC(pkt->getSSRC(),
 							created);
 				if( srcLink->getGoodbye() )
-					onGotGoodbye(*(srcLink->getSource()), NULL);
+					onGotGoodbye(*(srcLink->getSource()), "");
 				BYESource(pkt->getSSRC());
 			}
 			pointer += pkt->getLength();
@@ -834,13 +851,13 @@ QueueRTCPManager::dispatchControlPacket(void)
 	// (B) put report blocks
 	// After adding report blocks, we have to leave room for at
 	// least a CNAME SDES item
-	uint16 available = getPathMTU()
+	uint16 available = (uint16)(getPathMTU()
 		- lowerHeadersSize
 		- len
 		- (sizeof(RTCPFixedHeader) +
 		   2*sizeof(uint8) + 
 		   getApplication().getSDESItem(SDESItemTypeCNAME).length()) 
-		- 100;
+		- 100);
 
 	// if we have to go to a new RR packet
 	bool another = false;
@@ -887,7 +904,7 @@ QueueRTCPManager::dispatchControlPacket(void)
 	// fill the padding with 0s
 	packSDES(len);
 
-	// TODO: virtual for sending APP RTCP packets
+	// TODO: virtual for sending APP RTCP packets?
 
 	// actually send the packet.
 	size_t count = sendControlToDestinations(rtcpSendBuffer,len);
@@ -916,13 +933,13 @@ QueueRTCPManager::packSDES(uint16 &len)
 		getApplication().getSDESItem(SDESItemTypeCNAME).length();
 	const char* cname = 
 		getApplication().getSDESItem(SDESItemTypeCNAME).c_str();
-	pkt->info.SDES.item.len = cnameLen;
+	pkt->info.SDES.item.len = (uint8)cnameLen;
 	len += sizeof(RTCPFixedHeader) + sizeof(pkt->info.SDES.ssrc) + 
 		sizeof(pkt->info.SDES.item.type) +
 		sizeof(pkt->info.SDES.item.len);
 	
 	memcpy((rtcpSendBuffer + len),cname,cnameLen);
-	len += cnameLen;
+	len += (uint16)cnameLen;
 	// pack items other than CNAME (following priorities
 	// stablished inside scheduleSDESItem()).
 	SDESItemType nexttype = scheduleSDESItem();
@@ -932,7 +949,7 @@ QueueRTCPManager::packSDES(uint16 &len)
 		item->type = nexttype;
 		const char *content = 
 			getApplication().getSDESItem(nexttype).c_str();
-		item->len = strlen(content);
+		item->len = (uint8)strlen(content);
 		len += 2;
 		memcpy(reinterpret_cast<char *>(rtcpSendBuffer + len),
 			  content,item->len);
@@ -970,8 +987,7 @@ QueueRTCPManager::packReportBlocks(RRBlock* blocks, uint16 &len,
 		// update stats.
 		srcLink.computeStats();
 		blocks[j].ssrc = Nssrc;
-		blocks[j].rinfo.fractionLost = 
-			htonl(srcLink.getFractionLost());
+		blocks[j].rinfo.fractionLost = srcLink.getFractionLost();
 		blocks[j].rinfo.lostMSB = 
 			(srcLink.getCumulativePacketLost() & 0xFF0000) >> 16;
 		blocks[j].rinfo.lostLSW = 
@@ -1081,7 +1097,7 @@ QueueRTCPManager::sendControlToDestinations(unsigned char* buffer, size_t len)
 }
 
 #ifdef	CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
