@@ -14,29 +14,26 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 // 
-// As a special exception to the GNU General Public License, permission is 
-// granted for additional uses of the text contained in its release 
-// of ccRTP.
-// 
-// The exception is that, if you link the ccRTP library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the ccRTP library code into it.
-// 
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name ccRTP.  If you copy code from other releases into a copy of
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however    
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.    
+//
+// This exception applies only to the code released under the name GNU
+// ccRTP.  If you copy code from other releases into a copy of GNU
 // ccRTP, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for ccRTP, it is your choice
+//
+// If you write modifications of your own for GNU ccRTP, it is your choice
 // whether to permit this exception to apply to your modifications.
-// If you do not wish that, delete this exception notice.  
+// If you do not wish that, delete this exception notice.
+//
 
 #include "private.h"
 #include <ccrtp/oqueue.h>
@@ -67,9 +64,13 @@ DestinationListHandler::~DestinationListHandler()
 	while ( firstDestination ) {
 		tmp = getFirstDestination();
 		firstDestination = firstDestination->getNext();
+#ifdef	CCXX_EXCEPTIONS
 		try {
+#endif
 			delete tmp;
+#ifdef	CCXX_EXCEPTIONS
 		} catch (...) {}
+#endif
 	}
 	unlockDestinationList();
 }
@@ -108,8 +109,11 @@ DestinationListHandler::removeDestinationFromList(const InetAddress& ia,
 			result = true;
 			if ( prev )
 				prev->setNext(ta->getNext());
+			if ( getFirstDestination() == ta ) 
+				firstDestination = NULL;
 			destinationCounter--;
 			delete ta;
+			ta = NULL;
 		} else {
 			prev = ta;
 			ta = ta->getNext();
@@ -335,6 +339,8 @@ OutgoingDataQueue::putData(uint32 stamp, const unsigned char *data,
 
 	size_t step = 0, offset = 0;
 	while ( offset < datalen ) {
+		// remainder and step take care of segmentation
+		// according to getMaxSendSegmentSize()
 		size_t remainder = datalen - offset;
 		step = ( remainder > getMaxSendSegmentSize() ) ?
 			getMaxSendSegmentSize() : remainder;
@@ -383,39 +389,37 @@ OutgoingDataQueue::dispatchDataPacket(void)
 	}
 	
 	OutgoingRTPPkt* packet = packetLink->getPacket();
-	int32 rtn = packet->getPayloadSize();
-	if ( rtn ) {
-		lockDestinationList();
-		if ( isSingleDestination() ) {
+	uint32 rtn = packet->getPayloadSize();
+	lockDestinationList();
+	if ( isSingleDestination() ) {
+		sendData(packet->getRawPacket(),
+			 packet->getRawPacketSize());
+	} else {
+		// when no destination has been added, NULL == dest.
+		TransportAddress* dest = getFirstDestination();
+		while ( dest ) {
+			setDataPeer(dest->getNetworkAddress(),
+				    dest->getDataTransportPort());
 			sendData(packet->getRawPacket(),
 				 packet->getRawPacketSize());
-		} else {
-			// when no destination has been added, NULL == dest.
-			TransportAddress* dest = getFirstDestination();
-			while ( dest ) {
-				setDataPeer(dest->getNetworkAddress(),
-					    dest->getDataTransportPort());
-				sendData(packet->getRawPacket(),
-					 packet->getRawPacketSize());
-				dest = dest->getNext();
-			}
+			dest = dest->getNext();
 		}
-		unlockDestinationList();
 	}
+	unlockDestinationList();
+
 	// unlink the sent packet from the queue and destroy it. Also
 	// record the sending.
-	if ( rtn > -1 ) {
-		sendFirst = sendFirst->getNext();
-		if ( sendFirst ) {
-			sendFirst->setPrev(NULL);
-		} else {
-			sendLast = NULL;
-		}
-		// for general accounting and RTCP SR statistics 
-		sendInfo.packetCount++;
-		sendInfo.octetCount += packet->getPayloadSize();
-		delete packetLink;
+	sendFirst = sendFirst->getNext();
+	if ( sendFirst ) {
+		sendFirst->setPrev(NULL);
+	} else {
+		sendLast = NULL;
 	}
+	// for general accounting and RTCP SR statistics 
+	sendInfo.packetCount++;
+	sendInfo.octetCount += packet->getPayloadSize();
+	delete packetLink;
+
 	sendLock.unlock();
 	return rtn;
 }
@@ -456,7 +460,7 @@ OutgoingDataQueue::setPartial(uint32 stamp, unsigned char *data,
 }
 
 #ifdef  CCXX_NAMESPACES
-};
+}
 #endif
 
 /** EMACS **
